@@ -85,12 +85,11 @@ const uint8_t hid_descriptor_keyboard[] = {
 
     // Modifier byte (input)
 
-    0x05, 0x07,                    //   Usage Page (Key Codes)
     0x75, 0x01,                    //   Report Size (1)
     0x95, 0x08,                    //   Report Count (8)
     0x05, 0x07,                    //   Usage Page (Key codes)
     0x19, 0xe0,                    //   Usage Minimum (Keyboard LeftControl)
-    0x29, 0xe7,                    //   Usage Maxium (Keyboard Right GUI)
+    0x29, 0xe7,                    //   Usage Maximum (Keyboard Right GUI)
     0x15, 0x00,                    //   Logical Minimum (0)
     0x25, 0x01,                    //   Logical Maximum (1)
     0x81, 0x02,                    //   Input (Data, Variable, Absolute)
@@ -107,7 +106,7 @@ const uint8_t hid_descriptor_keyboard[] = {
     0x75, 0x01,                    //   Report Size (1)
     0x05, 0x08,                    //   Usage Page (LEDs)
     0x19, 0x01,                    //   Usage Minimum (Num Lock)
-    0x29, 0x05,                    //   Usage Maxium (Kana)
+    0x29, 0x05,                    //   Usage Maximum (Kana)
     0x91, 0x02,                    //   Output (Data, Variable, Absolute)
 
     0x95, 0x01,                    //   Report Count (1)
@@ -122,10 +121,10 @@ const uint8_t hid_descriptor_keyboard[] = {
     0x25, 0xff,                    //   Logical Maximum (1)
     0x05, 0x07,                    //   Usage Page (Key codes)
     0x19, 0x00,                    //   Usage Minimum (Reserved (no event indicated))
-    0x29, 0xff,                    //   Usage Maxium (Reserved)
+    0x29, 0xff,                    //   Usage Maximum (Reserved)
     0x81, 0x00,                    //   Input (Data, Array)
 
-    0xc0,                          // End collection  
+    0xc0,                          // End collection
 };
 
 // 
