@@ -163,8 +163,9 @@ extern "C" {
 #define HFP_RING "RING"
 
 // Codecs 
-#define HFP_CODEC_CVSD 0x01
-#define HFP_CODEC_MSBC 0x02
+#define HFP_CODEC_CVSD    0x01
+#define HFP_CODEC_MSBC    0x02
+#define HFP_CODEC_LC3_SWB 0x03
 
 typedef enum {
     HFP_ROLE_INVALID = 0,
@@ -546,6 +547,7 @@ typedef struct hfp_connection {
     bd_addr_t remote_addr;
     hci_con_handle_t acl_handle;
     hci_con_handle_t sco_handle;
+    uint16_t packet_types;
     uint8_t rfcomm_channel_nr;
     uint16_t rfcomm_cid;
     uint16_t rfcomm_mtu;
@@ -639,6 +641,9 @@ typedef struct hfp_connection {
     // HF: AT Command, AG: Unsolicited Result Code
     const char * send_custom_message;
 
+    // HF:  Unsolicited Result Code, AG:  AT Command
+    uint16_t custom_at_command_id;
+
     bool emit_vra_enabled_after_audio_established;
     // AG only
     uint8_t change_in_band_ring_tone_setting;
@@ -663,7 +668,6 @@ typedef struct hfp_connection {
 
     int send_status_of_current_calls;
     int next_call_index;
-    uint16_t ag_custom_at_command_id;
 
     // HF only
     // HF: track command for which ok/error response need to be received
@@ -758,8 +762,38 @@ int get_bit(uint16_t bitmap, int position);
 int store_bit(uint32_t bitmap, int position, uint8_t value);
 // UTILS_END
 
+#define HFP_H2_SYNC_FRAME_SIZE 60
+// HFP H2 Synchronization
+typedef struct {
+    // callback returns true if data was valid
+    bool        (*callback)(bool bad_frame, const uint8_t * frame_data, uint16_t frame_len);
+    uint8_t     frame_data[HFP_H2_SYNC_FRAME_SIZE];
+    uint16_t    frame_len;
+    uint16_t    dropped_bytes;
+} hfp_h2_sync_t;
+
+/**
+ * @brief Init HFP H2 Sync state
+ * @param hfp_h2_sync
+ * @param callback
+ */
+void hfp_h2_sync_init(hfp_h2_sync_t * hfp_h2_sync,
+                      bool (*callback)(bool bad_frame, const uint8_t * frame_data, uint16_t frame_len));
+/**
+ * @brief Process H2 data and execute callback for frames with valid H2 header
+ * @param hfp_h2_sync
+ * @param bad_frame
+ * @param frame_data
+ * @param frame_len
+ */
+void hfp_h2_sync_process(hfp_h2_sync_t *hfp_h2_sync, bool bad_frame, const uint8_t *frame_data, uint16_t frame_len);
+
+
+// other
+
 void hfp_finalize_connection_context(hfp_connection_t * hfp_connection);
-void hfp_emit_sco_event(hfp_connection_t * hfp_connection, uint8_t status, hci_con_handle_t con_handle, bd_addr_t addr, uint8_t  negotiated_codec);
+void hfp_emit_sco_connection_established(hfp_connection_t *hfp_connection, uint8_t status, uint8_t negotiated_codec,
+                                         uint16_t rx_packet_length, uint16_t tx_packet_length);
 
 void hfp_set_ag_callback(btstack_packet_handler_t callback);
 void hfp_set_ag_rfcomm_packet_handler(btstack_packet_handler_t handler);
@@ -771,6 +805,7 @@ void hfp_init(void);
 void hfp_deinit(void);
 
 void hfp_register_custom_ag_command(hfp_custom_at_command_t * at_command);
+void hfp_register_custom_hf_command(hfp_custom_at_command_t * at_command);
 
 void hfp_create_sdp_record(uint8_t * service, uint32_t service_record_handle, uint16_t service_uuid, int rfcomm_channel_nr, const char * name);
 void hfp_handle_hci_event(uint8_t packet_type, uint16_t channel, uint8_t *packet, uint16_t size, hfp_role_t local_role);
@@ -836,7 +871,10 @@ void hfp_accept_synchronous_connection(hfp_connection_t * hfp_connection, bool i
 int hfp_supports_codec(uint8_t codec, int codecs_nr, uint8_t * codecs);
 void hfp_hf_drop_mSBC_if_eSCO_not_supported(uint8_t * codecs, uint8_t * codecs_nr);
 void hfp_init_link_settings(hfp_connection_t * hfp_connection, uint8_t eSCO_S4_supported);
-hfp_link_settings_t hfp_next_link_setting(hfp_link_settings_t current_setting, bool local_eSCO_supported, bool remote_eSCO_supported, bool eSCO_s4_supported, uint8_t negotiated_codec);
+hfp_link_settings_t hfp_next_link_setting(hfp_link_settings_t current_setting, uint16_t local_sco_packet_types,
+                                          uint16_t remote_sco_packet_types, bool eSCO_s4_supported,
+                                          uint8_t negotiated_codec);
+hfp_link_settings_t hfp_safe_settings_for_context(bool use_eSCO, uint8_t negotiated_codec, bool secure_connection_in_use);
 
 const char * hfp_hf_feature(int index);
 const char * hfp_ag_feature(int index);
