@@ -2201,7 +2201,7 @@ const hci_cmd_t hci_le_reject_cis_request = {
  * @param broadcast_code
  */
 const hci_cmd_t hci_le_create_big = {
-    HCI_OPCODE_HCI_LE_CREATE_BIG, "11132211111P"
+    HCI_OPCODE_HCI_LE_CREATE_BIG, "11132211111K"
 };
 
 /**
@@ -2223,7 +2223,7 @@ const hci_cmd_t hci_le_create_big = {
  * @param broadcast_code
  */
 const hci_cmd_t hci_le_create_big_test = {
-    HCI_OPCODE_HCI_LE_CREATE_BIG_TEST, "111321221111111P"
+    HCI_OPCODE_HCI_LE_CREATE_BIG_TEST, "111321221111111K"
 };
 
 /**
@@ -2246,7 +2246,7 @@ const hci_cmd_t hci_le_terminate_big = {
  */
 
 const hci_cmd_t hci_le_big_create_sync = {
-    HCI_OPCODE_HCI_LE_BIG_CREATE_SYNC, "1H1P12a[1]"
+    HCI_OPCODE_HCI_LE_BIG_CREATE_SYNC, "1H1K12a[1]"
 };
 
 /**
@@ -2468,7 +2468,7 @@ const hci_cmd_t hci_bcm_set_tx_pwr = {
 /**
  * @brief This command starts receiving packets using packet transmission parameters such as
  *        frequency channel, packet type, and packet length. It is used for Packet RX.
- * @see   https://processors.wiki.ti.com/index.php/CC256x_Testing_Guide#Continuous_RX
+ * @see   processors.wiki.ti.com/index.php/CC256x_Testing_Guide#Continuous_RX
  * @param frequency
  * @param ADPLL loop mode
  */
@@ -2585,7 +2585,7 @@ const hci_cmd_t hci_ti_write_codec_config = {
 
 /**
  * @brief This command is used only for internal testing.
- * @see   https://processors.wiki.ti.com/index.php/CC256x_Testing_Guide#Continuous_TX
+ * @see   processors.wiki.ti.com/index.php/CC256x_Testing_Guide#Continuous_TX
  * @param frequency
  * @param ADPLL loop mode
  */
@@ -2596,7 +2596,7 @@ const hci_cmd_t hci_ti_drpb_enable_rf_calibration = {
 /**
  * @brief This command command is only required for the continuous TX test of modulated
  * (GFSK, π/4-DQPSK or 8DPSK) signal. This command should be skipped when performing continuous TX test for CW.
- * @see   https://processors.wiki.ti.com/index.php/CC256x_Testing_Guide#Continuous_RX
+ * @see   processors.wiki.ti.com/index.php/CC256x_Testing_Guide#Continuous_RX
  * @param frequency
  * @param ADPLL loop mode
  */
@@ -2608,5 +2608,12 @@ const hci_cmd_t hci_ti_write_hardware_register = {
  * @brief Configure SCO routing on Realtek Controllers
  */
 const hci_cmd_t hci_rtk_configure_sco_routing = {
-    HCI_OPCODE (0x3f, 0x93), "111111111"
+    HCI_OPCODE_HCI_RTK_CONFIGURE_SCO_ROUTING, "111111111"
+};
+
+/**
+* @brief Read Card Information
+*/
+const hci_cmd_t hci_rtk_read_card_info = {
+    HCI_OPCODE_HCI_RTK_READ_CARD_INFO, "11111"
 };
