@@ -1,5 +1,5 @@
 /***********************************************************************************************************************
- * Copyright [2020-2021] Renesas Electronics Corporation and/or its affiliates.  All Rights Reserved.
+ * Copyright [2020-2023] Renesas Electronics Corporation and/or its affiliates.  All Rights Reserved.
  *
  * This software and documentation are supplied by Renesas Electronics America Inc. and may only be used with products
  * of Renesas Electronics Corp. and its affiliates ("Renesas").  No other uses are authorized.  Renesas products are
@@ -50,8 +50,6 @@ FSP_HEADER
 /**********************************************************************************************************************
  * Macro definitions
  **********************************************************************************************************************/
-#define TRANSFER_API_VERSION_MAJOR            (1U) // DEPRECATED
-#define TRANSFER_API_VERSION_MINOR            (0U) // DEPRECATED
 
 #define TRANSFER_SETTINGS_MODE_BITS           (30U)
 #define TRANSFER_SETTINGS_SIZE_BITS           (28U)
@@ -93,7 +91,11 @@ typedef enum e_transfer_mode
      *  @ref transfer_addr_mode_t.  After the block transfer is complete, transfer_info_t::num_blocks is
      *  decremented.  After the transfer_info_t::num_blocks reaches 0, transfer requests will not cause any
      *  further transfers. */
-    TRANSFER_MODE_BLOCK = 2
+    TRANSFER_MODE_BLOCK = 2,
+
+    /** In addition to block mode features, repeat-block mode supports a ring buffer of blocks and offsets
+     *  within a block (to split blocks into arrays of their first data, second data, etc.) */
+    TRANSFER_MODE_REPEAT_BLOCK = 3
 } transfer_mode_t;
 
 /** Transfer size specifies the size of each individual transfer.
@@ -123,14 +125,14 @@ typedef enum e_transfer_addr_mode
 } transfer_addr_mode_t;
 
 /** Repeat area options (source or destination).  In @ref TRANSFER_MODE_REPEAT, the selected pointer returns to its
- *  original value after transfer_info_t::length transfers.  In @ref TRANSFER_MODE_BLOCK, the selected pointer
- *  returns to its original value after each transfer. */
+ *  original value after transfer_info_t::length transfers.  In @ref TRANSFER_MODE_BLOCK and @ref TRANSFER_MODE_REPEAT_BLOCK,
+ *  the selected pointer returns to its original value after each transfer. */
 typedef enum e_transfer_repeat_area
 {
-    /** Destination area repeated in @ref TRANSFER_MODE_REPEAT or @ref TRANSFER_MODE_BLOCK. */
+    /** Destination area repeated in @ref TRANSFER_MODE_REPEAT or @ref TRANSFER_MODE_BLOCK or @ref TRANSFER_MODE_REPEAT_BLOCK. */
     TRANSFER_REPEAT_AREA_DESTINATION = 0,
 
-    /** Source area repeated in @ref TRANSFER_MODE_REPEAT or @ref TRANSFER_MODE_BLOCK. */
+    /** Source area repeated in @ref TRANSFER_MODE_REPEAT or @ref TRANSFER_MODE_BLOCK or @ref TRANSFER_MODE_REPEAT_BLOCK. */
     TRANSFER_REPEAT_AREA_SOURCE = 1
 } transfer_repeat_area_t;
 
@@ -211,18 +213,21 @@ typedef struct st_transfer_info
 
             /** Select mode from @ref transfer_mode_t. */
             transfer_mode_t mode : 2;
-        };
+        } transfer_settings_word_b;
+
         uint32_t transfer_settings_word;
     };
 
     void const * volatile p_src;       ///< Source pointer
     void * volatile       p_dest;      ///< Destination pointer
 
-    /** Number of blocks to transfer when using @ref TRANSFER_MODE_BLOCK (both DTC an DMAC) and
-     * @ref TRANSFER_MODE_REPEAT (DMAC only), unused in other modes. */
+    /** Number of blocks to transfer when using @ref TRANSFER_MODE_BLOCK (both DTC an DMAC) or
+     * @ref TRANSFER_MODE_REPEAT (DMAC only) or
+     * @ref TRANSFER_MODE_REPEAT_BLOCK (DMAC only), unused in other modes. */
     volatile uint16_t num_blocks;
 
-    /** Length of each transfer.  Range limited for @ref TRANSFER_MODE_BLOCK and @ref TRANSFER_MODE_REPEAT,
+    /** Length of each transfer.  Range limited for @ref TRANSFER_MODE_BLOCK, @ref TRANSFER_MODE_REPEAT,
+     *  and @ref TRANSFER_MODE_REPEAT_BLOCK
      *  see HAL driver for details. */
     volatile uint16_t length;
 } transfer_info_t;
@@ -347,14 +352,6 @@ typedef struct st_transfer_api
      * @param[in]     p_ctrl    Control block set in @ref transfer_api_t::open call for this transfer.
      */
     fsp_err_t (* close)(transfer_ctrl_t * const p_ctrl);
-
-    /* DEPRECATED Gets version and stores it in provided pointer p_version.
-     * @par Implemented as
-     * - @ref R_DTC_VersionGet()
-     * - @ref R_DMAC_VersionGet()
-     * @param[out]    p_version  Code and API version used.
-     */
-    fsp_err_t (* versionGet)(fsp_version_t * const p_version);
 } transfer_api_t;
 
 /** This structure encompasses everything that is needed to use an instance of this interface. */
