@@ -1,5 +1,5 @@
 /***********************************************************************************************************************
- * Copyright [2020-2021] Renesas Electronics Corporation and/or its affiliates.  All Rights Reserved.
+ * Copyright [2020-2023] Renesas Electronics Corporation and/or its affiliates.  All Rights Reserved.
  *
  * This software and documentation are supplied by Renesas Electronics America Inc. and may only be used with products
  * of Renesas Electronics Corp. and its affiliates ("Renesas").  No other uses are authorized.  Renesas products are
@@ -47,10 +47,6 @@ FSP_HEADER
  * Macro definitions
  **********************************************************************************************************************/
 
-/* Version of code that implements the API defined in this file */
-#define ADC_CODE_VERSION_MAJOR                 (1U) // DEPRECATED
-#define ADC_CODE_VERSION_MINOR                 (1U) // DEPRECATED
-
 /* Typical values that can be used to modify the sample states.
  * The minimum sample state count value is either 6 or 7 depending on the clock ratios.
  * It is fixed to 7 based on the fact that at the lowest ADC conversion clock supported (1 MHz)
@@ -127,14 +123,34 @@ typedef enum e_adc_clear
     ADC_CLEAR_AFTER_READ_ON  = 1       ///< Clear after read on
 } adc_clear_t;
 
-/** ADC VREFAMPCNT config options
- * Reference Table 32.12 "VREFADC output voltage control list" in the RA2A1 manual R01UH0888EJ0100.*/
+/* VREF configuration options, not all options are available on all MCUs. If the MCU does not have VREFAMPCNT or
+ * ADHVREFCNT. */
 typedef enum e_adc_vref_control
 {
-    ADC_VREF_CONTROL_VREFH       = 0,  ///< VREFAMPCNT reset value. VREFADC Output voltage is Hi-Z
-    ADC_VREF_CONTROL_1_5V_OUTPUT = 25, ///< BGR turn ON. VREFADC Output voltage is 1.5 V
-    ADC_VREF_CONTROL_2_0V_OUTPUT = 29, ///< BGR turn ON. VREFADC Output voltage is 2.0 V
-    ADC_VREF_CONTROL_2_5V_OUTPUT = 31, ///< BGR turn ON. VREFADC Output voltage is 2.5 V
+    /* Available selections on MCUs with VREFAMPCNT.
+     * Reference Table 32.12 "VREFADC output voltage control list" in the RA2A1 manual R01UH0888EJ0100.*/
+
+    ADC_VREF_CONTROL_VREFH       = 0,      ///< VREFAMPCNT reset value. VREFADC Output voltage is Hi-Z
+    ADC_VREF_CONTROL_1_5V_OUTPUT = 25,     ///< BGR turn ON. VREFADC Output voltage is 1.5 V
+    ADC_VREF_CONTROL_2_0V_OUTPUT = 29,     ///< BGR turn ON. VREFADC Output voltage is 2.0 V
+    ADC_VREF_CONTROL_2_5V_OUTPUT = 31,     ///< BGR turn ON. VREFADC Output voltage is 2.5 V
+
+    /* Available selections on MCUs with ADHVREFCNT.
+     * Reference Section 35.2.31 "A/D High-Potential/Low-Potential Reference Voltage Control Register (ADHVREFCNT)"
+     * in the RA4M1 manual R01UH0887EJ0100.*/
+
+    ADC_VREF_CONTROL_AVCC0_AVSS0  = 0x0,   ///< High potential is AVCC0, low potential is AVSS0
+    ADC_VREF_CONTROL_VREFH0_AVSS0 = 0x1,   ///< High potential is VREFH0, low potential is AVSS0
+
+    /** High potential is internal reference voltage, low potential is AVSS0. When the high potential is set to the
+     * internal reference voltage, wait 5 us after R_ADC_Open() to start an ADC measurement. */
+    ADC_VREF_CONTROL_IVREF_AVSS0   = 0x2,
+    ADC_VREF_CONTROL_AVCC0_VREFL0  = 0x10, ///< High potential is AVCC0, low potential is VREFL0
+    ADC_VREF_CONTROL_VREFH0_VREFL0 = 0x11, ///< High potential is VREFH0, low potential is VREFL0
+
+    /** High potential is internal reference voltage, low potential is VREFL0. When the high potential is set to the
+     * internal reference voltage, wait 5 us after R_ADC_Open() to start an ADC measurement. */
+    ADC_VREF_CONTROL_IVREF_VREFL0 = 0x12,
 } adc_vref_control_t;
 
 /** ADC sample state registers */
@@ -159,6 +175,59 @@ typedef enum e_adc_sample_state_reg
     ADC_SAMPLE_STATE_CHANNEL_16_TO_31 = -3, ///< Sample state register channel 16 to 31
 } adc_sample_state_reg_t;
 
+/** ADC comparison settings */
+typedef enum e_adc_compare_cfg
+{
+    ADC_COMPARE_CFG_EVENT_OUTPUT_OR  = 0,
+    ADC_COMPARE_CFG_EVENT_OUTPUT_XOR = 1,
+    ADC_COMPARE_CFG_EVENT_OUTPUT_AND = 2,
+    ADC_COMPARE_CFG_A_ENABLE         = R_ADC0_ADCMPCR_CMPAE_Msk | R_ADC0_ADCMPCR_CMPAIE_Msk,
+    ADC_COMPARE_CFG_B_ENABLE         = R_ADC0_ADCMPCR_CMPBE_Msk | R_ADC0_ADCMPCR_CMPBIE_Msk,
+    ADC_COMPARE_CFG_WINDOW_ENABLE    = R_ADC0_ADCMPCR_WCMPE_Msk,
+} adc_compare_cfg_t;
+
+/** ADC Window B channel */
+typedef enum e_adc_window_b_channel
+{
+    ADC_WINDOW_B_CHANNEL_0 = 0,
+    ADC_WINDOW_B_CHANNEL_1,
+    ADC_WINDOW_B_CHANNEL_2,
+    ADC_WINDOW_B_CHANNEL_3,
+    ADC_WINDOW_B_CHANNEL_4,
+    ADC_WINDOW_B_CHANNEL_5,
+    ADC_WINDOW_B_CHANNEL_6,
+    ADC_WINDOW_B_CHANNEL_7,
+    ADC_WINDOW_B_CHANNEL_8,
+    ADC_WINDOW_B_CHANNEL_9,
+    ADC_WINDOW_B_CHANNEL_10,
+    ADC_WINDOW_B_CHANNEL_11,
+    ADC_WINDOW_B_CHANNEL_12,
+    ADC_WINDOW_B_CHANNEL_13,
+    ADC_WINDOW_B_CHANNEL_14,
+    ADC_WINDOW_B_CHANNEL_15,
+    ADC_WINDOW_B_CHANNEL_16,
+    ADC_WINDOW_B_CHANNEL_17,
+    ADC_WINDOW_B_CHANNEL_18,
+    ADC_WINDOW_B_CHANNEL_19,
+    ADC_WINDOW_B_CHANNEL_20,
+    ADC_WINDOW_B_CHANNEL_21,
+    ADC_WINDOW_B_CHANNEL_22,
+    ADC_WINDOW_B_CHANNEL_23,
+    ADC_WINDOW_B_CHANNEL_24,
+    ADC_WINDOW_B_CHANNEL_25,
+    ADC_WINDOW_B_CHANNEL_26,
+    ADC_WINDOW_B_CHANNEL_27,
+    ADC_WINDOW_B_CHANNEL_TEMPERATURE = 32,
+    ADC_WINDOW_B_CHANNEL_VOLT        = 33,
+} adc_window_b_channel_t;
+
+/** ADC Window B comparison mode */
+typedef enum e_adc_window_b_mode
+{
+    ADC_WINDOW_B_MODE_LESS_THAN_OR_OUTSIDE   = 0,
+    ADC_WINDOW_B_MODE_GREATER_THAN_OR_INSIDE = R_ADC0_ADCMPBNSR_CMPLB_Msk,
+} adc_window_b_mode_t;
+
 /** ADC action for group A interrupts group B scan.
  * This enumeration is used to specify the priority between Group A and B in group mode.  */
 typedef enum e_adc_group_a
@@ -184,6 +253,20 @@ typedef struct st_adc_sample_state
     uint8_t                num_states; ///< Number of sampling states for conversion. Ch16-20/21 use the same value.
 } adc_sample_state_t;
 
+/** ADC Window Compare configuration */
+typedef struct st_adc_window_cfg
+{
+    uint32_t               compare_mask;       ///< Channel mask to compare with Window A
+    uint32_t               compare_mode_mask;  ///< Per-channel condition mask for Window A
+    adc_compare_cfg_t      compare_cfg;        ///< Window Compare configuration
+    uint16_t               compare_ref_low;    ///< Window A lower reference value
+    uint16_t               compare_ref_high;   ///< Window A upper reference value
+    uint16_t               compare_b_ref_low;  ///< Window B lower reference value
+    uint16_t               compare_b_ref_high; ///< Window A upper reference value
+    adc_window_b_channel_t compare_b_channel;  ///< Window B channel
+    adc_window_b_mode_t    compare_b_mode;     ///< Window B condition setting
+} adc_window_cfg_t;
+
 /** Extended configuration structure for ADC. */
 typedef struct st_adc_extended_cfg
 {
@@ -192,17 +275,23 @@ typedef struct st_adc_extended_cfg
     adc_trigger_t        trigger_group_b;     ///< Group B trigger source; valid only for group mode
     adc_double_trigger_t double_trigger_mode; ///< Double-trigger mode setting
     adc_vref_control_t   adc_vref_control;    ///< VREFADC output voltage control
+    uint8_t              enable_adbuf;        ///< Enable ADC Ring Buffer, Valid only to use along with DMAC transfer
+    IRQn_Type            window_a_irq;        ///< IRQ number for Window Compare A interrupts
+    IRQn_Type            window_b_irq;        ///< IRQ number for Window Compare B interrupts
+    uint8_t              window_a_ipl;        ///< Priority for Window Compare A interrupts
+    uint8_t              window_b_ipl;        ///< Priority for Window Compare B interrupts
 } adc_extended_cfg_t;
 
 /** ADC channel(s) configuration       */
 typedef struct st_adc_channel_cfg
 {
-    uint32_t      scan_mask;           ///< Channels/bits: bit 0 is ch0; bit 15 is ch15.
-    uint32_t      scan_mask_group_b;   ///< Valid for group modes.
-    uint32_t      add_mask;            ///< Valid if add enabled in Open().
-    adc_group_a_t priority_group_a;    ///< Valid for group modes.
-    uint8_t       sample_hold_mask;    ///< Channels/bits 0-2.
-    uint8_t       sample_hold_states;  ///< Number of states to be used for sample and hold. Affects channels 0-2.
+    uint32_t           scan_mask;          ///< Channels/bits: bit 0 is ch0; bit 15 is ch15.
+    uint32_t           scan_mask_group_b;  ///< Valid for group modes.
+    uint32_t           add_mask;           ///< Valid if add enabled in Open().
+    adc_window_cfg_t * p_window_cfg;       ///< Pointer to Window Compare configuration
+    adc_group_a_t      priority_group_a;   ///< Valid for group modes.
+    uint8_t            sample_hold_mask;   ///< Channels/bits 0-2.
+    uint8_t            sample_hold_states; ///< Number of states to be used for sample and hold. Affects channels 0-2.
 } adc_channel_cfg_t;
 
 /* Sample and hold Channel mask. Sample and hold is only available for channel 0,1,2*/
@@ -218,7 +307,8 @@ typedef struct
     R_ADC0_Type     * p_reg;                    // Base register for this unit
     adc_cfg_t const * p_cfg;
     uint32_t          opened;                   // Boolean to verify that the Unit has been initialized
-    uint32_t          scan_mask;                // Scan mask used for Normal scan.
+    uint32_t          initialized;              // Initialized status of ADC
+    uint32_t          scan_mask;                // Scan mask used for Normal scan
     uint16_t          scan_start_adcsr;
 
     void (* p_callback)(adc_callback_args_t *); // Pointer to callback that is called when an adc_event_t occurs.
@@ -245,6 +335,7 @@ fsp_err_t R_ADC_Open(adc_ctrl_t * p_ctrl, adc_cfg_t const * const p_cfg);
 fsp_err_t R_ADC_ScanCfg(adc_ctrl_t * p_ctrl, void const * const p_channel_cfg);
 fsp_err_t R_ADC_InfoGet(adc_ctrl_t * p_ctrl, adc_info_t * p_adc_info);
 fsp_err_t R_ADC_ScanStart(adc_ctrl_t * p_ctrl);
+fsp_err_t R_ADC_ScanGroupStart(adc_ctrl_t * p_ctrl, adc_group_mask_t group_id);
 fsp_err_t R_ADC_ScanStop(adc_ctrl_t * p_ctrl);
 fsp_err_t R_ADC_StatusGet(adc_ctrl_t * p_ctrl, adc_status_t * p_status);
 fsp_err_t R_ADC_Read(adc_ctrl_t * p_ctrl, adc_channel_t const reg_id, uint16_t * const p_data);
@@ -252,8 +343,7 @@ fsp_err_t R_ADC_Read32(adc_ctrl_t * p_ctrl, adc_channel_t const reg_id, uint32_t
 fsp_err_t R_ADC_SampleStateCountSet(adc_ctrl_t * p_ctrl, adc_sample_state_t * p_sample);
 fsp_err_t R_ADC_Close(adc_ctrl_t * p_ctrl);
 fsp_err_t R_ADC_OffsetSet(adc_ctrl_t * const p_ctrl, adc_channel_t const reg_id, int32_t offset);
-fsp_err_t R_ADC_Calibrate(adc_ctrl_t * const p_ctrl, void * const p_extend);
-fsp_err_t R_ADC_VersionGet(fsp_version_t * const p_version);
+fsp_err_t R_ADC_Calibrate(adc_ctrl_t * const p_ctrl, void const * p_extend);
 fsp_err_t R_ADC_CallbackSet(adc_ctrl_t * const          p_api_ctrl,
                             void (                    * p_callback)(adc_callback_args_t *),
                             void const * const          p_context,
