@@ -1,5 +1,5 @@
 /***********************************************************************************************************************
- * Copyright [2020-2021] Renesas Electronics Corporation and/or its affiliates.  All Rights Reserved.
+ * Copyright [2020-2023] Renesas Electronics Corporation and/or its affiliates.  All Rights Reserved.
  *
  * This software and documentation are supplied by Renesas Electronics America Inc. and may only be used with products
  * of Renesas Electronics Corp. and its affiliates ("Renesas").  No other uses are authorized.  Renesas products are
@@ -46,10 +46,6 @@
 /* Shift to get port in bsp_io_port_t and bsp_io_port_pin_t enums. */
 #define IOPORT_PRV_PORT_OFFSET            (8U)
 
-#ifndef BSP_MCU_VBATT_SUPPORT
- #define BSP_MCU_VBATT_SUPPORT            (0U)
-#endif
-
 #define IOPORT_PRV_PORT_BITS              (0xFF00U)
 #define IOPORT_PRV_PIN_BITS               (0x00FFU)
 
@@ -84,7 +80,7 @@ static void r_ioport_hw_pin_event_output_data_write(bsp_io_port_t  port,
 
 static void r_ioport_pfs_write(bsp_io_port_pin_t pin, uint32_t value);
 
-#if BSP_MCU_VBATT_SUPPORT
+#if BSP_FEATURE_SYSC_HAS_VBTICTLR || BSP_FEATURE_RTC_HAS_TCEN
 static void bsp_vbatt_init(ioport_cfg_t const * const p_pin_cfg); // Used internally by BSP
 
 #endif
@@ -93,15 +89,6 @@ static void bsp_vbatt_init(ioport_cfg_t const * const p_pin_cfg); // Used intern
  * Private global variables
  **********************************************************************************************************************/
 
-/* Version data structure used by error logger macro. */
-static const fsp_version_t g_ioport_version =
-{
-    .api_version_minor  = IOPORT_API_VERSION_MINOR,
-    .api_version_major  = IOPORT_API_VERSION_MAJOR,
-    .code_version_major = IOPORT_CODE_VERSION_MAJOR,
-    .code_version_minor = IOPORT_CODE_VERSION_MINOR
-};
-
 /***********************************************************************************************************************
  * Global Variables
  **********************************************************************************************************************/
@@ -115,7 +102,6 @@ const ioport_api_t g_ioport_on_ioport =
     .pinCfg               = R_IOPORT_PinCfg,
     .pinEventInputRead    = R_IOPORT_PinEventInputRead,
     .pinEventOutputWrite  = R_IOPORT_PinEventOutputWrite,
-    .pinEthernetModeCfg   = R_IOPORT_EthernetModeCfg,
     .pinRead              = R_IOPORT_PinRead,
     .pinWrite             = R_IOPORT_PinWrite,
     .portDirectionSet     = R_IOPORT_PortDirectionSet,
@@ -123,10 +109,9 @@ const ioport_api_t g_ioport_on_ioport =
     .portEventOutputWrite = R_IOPORT_PortEventOutputWrite,
     .portRead             = R_IOPORT_PortRead,
     .portWrite            = R_IOPORT_PortWrite,
-    .versionGet           = R_IOPORT_VersionGet,
 };
 
-#if BSP_MCU_VBATT_SUPPORT
+#if BSP_FEATURE_SYSC_HAS_VBTICTLR || BSP_FEATURE_RTC_HAS_TCEN
 static const bsp_io_port_pin_t g_vbatt_pins_input[] =
 {
     BSP_IO_PORT_04_PIN_02,             ///< Associated with VBTICTLR->VCH0INEN
@@ -248,7 +233,7 @@ fsp_err_t R_IOPORT_PinCfg (ioport_ctrl_t * const p_ctrl, bsp_io_port_pin_t pin,
     FSP_PARAMETER_NOT_USED(p_ctrl);
 #endif
 
-#if BSP_MCU_VBATT_SUPPORT
+#if BSP_FEATURE_SYSC_HAS_VBTICTLR || BSP_FEATURE_RTC_HAS_TCEN
 
     /* Create temporary structure for handling VBATT pins. */
     ioport_cfg_t     temp_cfg;
@@ -450,10 +435,10 @@ fsp_err_t R_IOPORT_PortDirectionSet (ioport_ctrl_t * const p_ctrl,
                                      ioport_size_t         direction_values,
                                      ioport_size_t         mask)
 {
-    ioport_size_t orig_value;
-    ioport_size_t set_bits;
-    ioport_size_t clr_bits;
-    ioport_size_t write_value;
+    uint32_t orig_value;
+    uint32_t set_bits;
+    uint32_t clr_bits;
+    uint32_t write_value;
 
 #if (1 == IOPORT_CFG_PARAM_CHECKING_ENABLE)
     ioport_instance_ctrl_t * p_instance_ctrl = (ioport_instance_ctrl_t *) p_ctrl;
@@ -468,23 +453,23 @@ fsp_err_t R_IOPORT_PortDirectionSet (ioport_ctrl_t * const p_ctrl,
     R_PORT0_Type * p_ioport_regs = IOPORT_PRV_PORT_ADDRESS((port >> IOPORT_PRV_PORT_OFFSET) & IOPORT_PRV_8BIT_MASK);
 
     /* Read current value of PCNTR1 register for the specified port */
-    orig_value = p_ioport_regs->PCNTR1 & IOPORT_PRV_16BIT_MASK;
+    orig_value = p_ioport_regs->PCNTR1;
 
     /* High bits */
     set_bits = direction_values & mask;
 
     /*  Low bits */
     /* Cast to ensure size */
-    clr_bits = (ioport_size_t) ((~direction_values) & mask);
+    clr_bits = (uint32_t) ((~direction_values) & mask);
 
     /* New value to write to port direction register */
     write_value  = orig_value;
     write_value |= set_bits;
 
-    /* Cast to ensure size */
-    write_value &= (ioport_size_t) (~clr_bits);
+    /* Clear bits as needed */
+    write_value &= ~clr_bits;
 
-    p_ioport_regs->PCNTR1 = write_value & IOPORT_PRV_16BIT_MASK;
+    p_ioport_regs->PCNTR1 = write_value;
 
     return FSP_SUCCESS;
 }
@@ -514,7 +499,7 @@ fsp_err_t R_IOPORT_PortEventInputRead (ioport_ctrl_t * const p_ctrl, bsp_io_port
     FSP_ERROR_RETURN(IOPORT_OPEN == p_instance_ctrl->open, FSP_ERR_NOT_OPEN);
     FSP_ASSERT(NULL != p_event_data);
     uint32_t port_number = port >> IOPORT_PRV_PORT_OFFSET;
-    FSP_ERROR_RETURN((port_number != 0) && (port_number <= BSP_FEATURE_IOPORT_ELC_PORTS), FSP_ERR_INVALID_ARGUMENT);
+    FSP_ERROR_RETURN((BSP_FEATURE_IOPORT_ELC_PORTS & (1 << port_number)), FSP_ERR_INVALID_ARGUMENT);
 #else
     FSP_PARAMETER_NOT_USED(p_ctrl);
 #endif
@@ -553,7 +538,7 @@ fsp_err_t R_IOPORT_PinEventInputRead (ioport_ctrl_t * const p_ctrl, bsp_io_port_
     FSP_ERROR_RETURN(IOPORT_OPEN == p_instance_ctrl->open, FSP_ERR_NOT_OPEN);
     FSP_ASSERT(NULL != p_pin_event);
     uint32_t port_number = pin >> IOPORT_PRV_PORT_OFFSET;
-    FSP_ERROR_RETURN((port_number != 0) && (port_number <= BSP_FEATURE_IOPORT_ELC_PORTS), FSP_ERR_INVALID_ARGUMENT);
+    FSP_ERROR_RETURN((BSP_FEATURE_IOPORT_ELC_PORTS & (1 << port_number)), FSP_ERR_INVALID_ARGUMENT);
 #else
     FSP_PARAMETER_NOT_USED(p_ctrl);
 #endif
@@ -609,7 +594,7 @@ fsp_err_t R_IOPORT_PortEventOutputWrite (ioport_ctrl_t * const p_ctrl,
     FSP_ERROR_RETURN(IOPORT_OPEN == p_instance_ctrl->open, FSP_ERR_NOT_OPEN);
     FSP_ERROR_RETURN(mask_value > (ioport_size_t) 0, FSP_ERR_INVALID_ARGUMENT);
     uint32_t port_number = port >> IOPORT_PRV_PORT_OFFSET;
-    FSP_ERROR_RETURN((port_number != 0) && (port_number <= BSP_FEATURE_IOPORT_ELC_PORTS), FSP_ERR_INVALID_ARGUMENT);
+    FSP_ERROR_RETURN((BSP_FEATURE_IOPORT_ELC_PORTS & (1 << port_number)), FSP_ERR_INVALID_ARGUMENT);
 #else
     FSP_PARAMETER_NOT_USED(p_ctrl);
 #endif
@@ -650,7 +635,7 @@ fsp_err_t R_IOPORT_PinEventOutputWrite (ioport_ctrl_t * const p_ctrl, bsp_io_por
     FSP_ERROR_RETURN(IOPORT_OPEN == p_instance_ctrl->open, FSP_ERR_NOT_OPEN);
     FSP_ERROR_RETURN((pin_value == BSP_IO_LEVEL_HIGH) || (pin_value == BSP_IO_LEVEL_LOW), FSP_ERR_INVALID_ARGUMENT);
     uint32_t port_number = pin >> IOPORT_PRV_PORT_OFFSET;
-    FSP_ERROR_RETURN((port_number != 0) && (port_number <= BSP_FEATURE_IOPORT_ELC_PORTS), FSP_ERR_INVALID_ARGUMENT);
+    FSP_ERROR_RETURN((BSP_FEATURE_IOPORT_ELC_PORTS & (1 << port_number)), FSP_ERR_INVALID_ARGUMENT);
 #else
     FSP_PARAMETER_NOT_USED(p_ctrl);
 #endif
@@ -683,60 +668,6 @@ fsp_err_t R_IOPORT_PinEventOutputWrite (ioport_ctrl_t * const p_ctrl, bsp_io_por
     return FSP_SUCCESS;
 }
 
-/***********************************************************************************************************************
- * DEPRECATED Returns IOPort HAL driver version. Implements @ref ioport_api_t::versionGet.
- *
- * @retval FSP_SUCCESS        Version information read
- * @retval FSP_ERR_ASSERTION  The parameter p_data is NULL
- *
- * @note This function is reentrant.
- **********************************************************************************************************************/
-fsp_err_t R_IOPORT_VersionGet (fsp_version_t * p_data)
-{
-#if (1 == IOPORT_CFG_PARAM_CHECKING_ENABLE)
-
-    /* Verify parameters are valid */
-    FSP_ASSERT(NULL != p_data);
-#endif
-
-    *p_data = g_ioport_version;
-
-    return FSP_SUCCESS;
-}
-
-/*******************************************************************************************************************//**
- * Configures Ethernet channel PHY mode. Implements @ref ioport_api_t::pinEthernetModeCfg.
- *
- * @retval FSP_SUCCESS              Ethernet PHY mode set
- * @retval FSP_ERR_INVALID_ARGUMENT Channel or mode not valid
- * @retval FSP_ERR_UNSUPPORTED      Ethernet configuration not supported on this device.
- * @retval FSP_ERR_NOT_OPEN         The module has not been opened
- * @retval FSP_ERR_ASSERTION        NULL pointer
- *
- * @note This function is not re-entrant.
- **********************************************************************************************************************/
-fsp_err_t R_IOPORT_EthernetModeCfg (ioport_ctrl_t * const     p_ctrl,
-                                    ioport_ethernet_channel_t channel,
-                                    ioport_ethernet_mode_t    mode)
-{
-    FSP_ERROR_RETURN(1U == BSP_FEATURE_IOPORT_HAS_ETHERNET, FSP_ERR_UNSUPPORTED);
-
-#if (1 == IOPORT_CFG_PARAM_CHECKING_ENABLE)
-    ioport_instance_ctrl_t * p_instance_ctrl = (ioport_instance_ctrl_t *) p_ctrl;
-    FSP_ASSERT(NULL != p_instance_ctrl);
-    FSP_ERROR_RETURN(IOPORT_OPEN == p_instance_ctrl->open, FSP_ERR_NOT_OPEN);
-    FSP_ERROR_RETURN(channel < IOPORT_ETHERNET_CHANNEL_END, FSP_ERR_INVALID_ARGUMENT);
-    FSP_ERROR_RETURN(mode < IOPORT_ETHERNET_MODE_END, FSP_ERR_INVALID_ARGUMENT);
-#else
-    FSP_PARAMETER_NOT_USED(p_ctrl);
-    FSP_PARAMETER_NOT_USED(channel);
-#endif
-
-    R_PMISC->PFENET = (uint8_t) mode;
-
-    return FSP_SUCCESS;
-}
-
 /*******************************************************************************************************************//**
  * @} (end addtogroup IOPORT)
  **********************************************************************************************************************/
@@ -752,7 +683,7 @@ fsp_err_t R_IOPORT_EthernetModeCfg (ioport_ctrl_t * const     p_ctrl,
  **********************************************************************************************************************/
 void r_ioport_pins_config (const ioport_cfg_t * p_cfg)
 {
-#if BSP_MCU_VBATT_SUPPORT
+#if BSP_FEATURE_SYSC_HAS_VBTICTLR || BSP_FEATURE_RTC_HAS_TCEN
 
     /* Handle any VBATT domain pin configuration. */
     bsp_vbatt_init(p_cfg);
@@ -843,7 +774,7 @@ static void r_ioport_pfs_write (bsp_io_port_pin_t pin, uint32_t value)
     R_PFS->PORT[pin >> IOPORT_PRV_PORT_OFFSET].PIN[pin & BSP_IO_PRV_8BIT_MASK].PmnPFS = value;
 }
 
-#if BSP_MCU_VBATT_SUPPORT
+#if BSP_FEATURE_SYSC_HAS_VBTICTLR || BSP_FEATURE_RTC_HAS_TCEN
 
 /*******************************************************************************************************************//**
  * @brief Initializes VBTICTLR register based on pin configuration.
@@ -857,12 +788,6 @@ static void bsp_vbatt_init (ioport_cfg_t const * const p_pin_cfg)
 {
     uint32_t pin_index;
     uint32_t vbatt_index;
-    uint8_t  local_vbtictlr_set;       ///< Will hold bits to set in VBTICTLR
-    uint8_t  local_vbtictlr_clear;     ///< Will hold bits to clear in VBTICTLR
-
-    /* Make no changes unless required. */
-    local_vbtictlr_set   = 0U;
-    local_vbtictlr_clear = 0U;
 
     /* Must loop over all pins as pin configuration table is unordered. */
     for (pin_index = 0U; pin_index < p_pin_cfg->number_of_pins; pin_index++)
@@ -881,29 +806,78 @@ static void bsp_vbatt_init (ioport_cfg_t const * const p_pin_cfg)
                 if ((IOPORT_PERIPHERAL_AGT == pfs_psel_value) || (IOPORT_PERIPHERAL_CLKOUT_COMP_RTC == pfs_psel_value))
                 {
                     /* Bit should be set to 1. */
-                    local_vbtictlr_set |= (uint8_t) (1U << vbatt_index);
+ #if BSP_FEATURE_SYSC_HAS_VBTICTLR
+                    if (0 == (R_SYSTEM->VBTICTLR & (uint8_t) (1U << vbatt_index)))
+                    {
+                        R_BSP_RegisterProtectDisable(BSP_REG_PROTECT_OM_LPC_BATT);
+                        R_SYSTEM->VBTICTLR |= (uint8_t) (1U << vbatt_index);
+                        R_BSP_RegisterProtectEnable(BSP_REG_PROTECT_OM_LPC_BATT);
+                    }
+                    else
+                    {
+                        /* Do nothing: it is already enabled. */
+                    }
+ #endif
+ #if BSP_FEATURE_RTC_HAS_TCEN
+  #if BSP_TZ_NONSECURE_BUILD
+                    if (0 == R_PSCU->PSARE_b.PSARE2)
+                    {
+                        /* Do nothing: non secure build can't configure secure RTC registers. */
+                    }
+                    else
+  #endif
+                    {
+                        if (0 == R_RTC->RTCCR[vbatt_index].RTCCR_b.TCEN)
+                        {
+                            R_RTC->RTCCR[vbatt_index].RTCCR_b.TCEN = 1;
+                            R_BSP_SoftwareDelay(BSP_PRV_RTC_RESET_DELAY_US, BSP_DELAY_UNITS_MICROSECONDS);
+                        }
+                        else
+                        {
+                            /* Do nothing: it is already enabled. */
+                        }
+                    }
+ #endif
                 }
                 else
                 {
                     /* Bit should be cleared to 0. */
-                    local_vbtictlr_clear |= (uint8_t) (1U << vbatt_index);
+ #if BSP_FEATURE_SYSC_HAS_VBTICTLR
+                    if ((R_SYSTEM->VBTICTLR & (uint8_t) (1U << vbatt_index)) > 0)
+                    {
+                        R_BSP_RegisterProtectDisable(BSP_REG_PROTECT_OM_LPC_BATT);
+                        R_SYSTEM->VBTICTLR &= (uint8_t) ~(1U << vbatt_index);
+                        R_BSP_RegisterProtectEnable(BSP_REG_PROTECT_OM_LPC_BATT);
+                    }
+                    else
+                    {
+                        /* Do nothing: it is already disabled. */
+                    }
+ #endif
+ #if BSP_FEATURE_RTC_HAS_TCEN
+  #if BSP_TZ_NONSECURE_BUILD
+                    if (0 == R_PSCU->PSARE_b.PSARE2)
+                    {
+                        /* Do nothing: non secure build can't configure secure RTC registers. */
+                    }
+                    else
+  #endif
+                    {
+                        if (R_RTC->RTCCR[vbatt_index].RTCCR_b.TCEN > 0)
+                        {
+                            R_RTC->RTCCR[vbatt_index].RTCCR_b.TCEN = 0;
+                            R_BSP_SoftwareDelay(BSP_PRV_RTC_RESET_DELAY_US, BSP_DELAY_UNITS_MICROSECONDS);
+                        }
+                        else
+                        {
+                            /* Do nothing: it is already disabled. */
+                        }
+                    }
+ #endif
                 }
             }
         }
     }
-
-    /* Disable write protection on VBTICTLR. */
-    R_BSP_RegisterProtectDisable(BSP_REG_PROTECT_OM_LPC_BATT);
-
-    /* Read value, set and clear bits as needed and write back. */
-    uint8_t local_vbtictlr = R_SYSTEM->VBTICTLR;
-    local_vbtictlr |= local_vbtictlr_set;              ///< Set appropriate bits
-    local_vbtictlr &= (uint8_t) ~local_vbtictlr_clear; ///< Clear appropriate bits
-
-    R_SYSTEM->VBTICTLR = local_vbtictlr;
-
-    /* Enable write protection on VBTICTLR. */
-    R_BSP_RegisterProtectEnable(BSP_REG_PROTECT_OM_LPC_BATT);
 }
 
 #endif
