@@ -1,5 +1,5 @@
 /***********************************************************************************************************************
- * Copyright [2020-2021] Renesas Electronics Corporation and/or its affiliates.  All Rights Reserved.
+ * Copyright [2020-2023] Renesas Electronics Corporation and/or its affiliates.  All Rights Reserved.
  *
  * This software and documentation are supplied by Renesas Electronics America Inc. and may only be used with products
  * of Renesas Electronics Corp. and its affiliates ("Renesas").  No other uses are authorized.  Renesas products are
@@ -146,15 +146,6 @@ fsp_err_t r_ssi_rx_unload_fifo(ssi_instance_ctrl_t * const p_instance_ctrl, void
  * Private global variables
  **********************************************************************************************************************/
 
-/* Version data structure used by error logger macro. */
-static const fsp_version_t g_version =
-{
-    .api_version_minor  = I2S_API_VERSION_MINOR,
-    .api_version_major  = I2S_API_VERSION_MAJOR,
-    .code_version_major = SSI_CODE_VERSION_MAJOR,
-    .code_version_minor = SSI_CODE_VERSION_MINOR
-};
-
 /***********************************************************************************************************************
  * Global Variables
  **********************************************************************************************************************/
@@ -170,7 +161,6 @@ const i2s_api_t g_i2s_on_ssi =
     .mute        = R_SSI_Mute,
     .statusGet   = R_SSI_StatusGet,
     .close       = R_SSI_Close,
-    .versionGet  = R_SSI_VersionGet,
     .callbackSet = R_SSI_CallbackSet,
 };
 
@@ -363,9 +353,8 @@ fsp_err_t R_SSI_Write (i2s_ctrl_t * const p_ctrl, void const * const p_src, uint
     FSP_ERROR_RETURN(SSI_PRV_OPEN == p_instance_ctrl->open, FSP_ERR_NOT_OPEN);
     FSP_ASSERT(NULL != p_src);
 
-    /* bytes must be a non-zero multiple of the FIFO access size multiplied by 2 (for the left and right channel). */
+    /* bytes must be a non-zero */
     FSP_ASSERT(bytes > 0U);
-    FSP_ASSERT(0U == (bytes % (2U << p_instance_ctrl->fifo_access_size)));
 #endif
 
     /* If a transfer instance is provided for write, reset the transfer. Otherwise store data to transmit in the
@@ -413,9 +402,8 @@ fsp_err_t R_SSI_Read (i2s_ctrl_t * const p_ctrl, void * const p_dest, uint32_t c
     FSP_ERROR_RETURN(SSI_PRV_OPEN == p_instance_ctrl->open, FSP_ERR_NOT_OPEN);
     FSP_ASSERT(NULL != p_dest);
 
-    /* bytes must be a non-zero multiple of the FIFO access size multiplied by 2 (for the left and right channel). */
+    /* bytes must be a non-zero */
     FSP_ASSERT(bytes > 0U);
-    FSP_ASSERT(0U == (bytes % (2U << p_instance_ctrl->fifo_access_size)));
 #endif
 
     /* If a transfer instance is provided for read, reset the transfer. Otherwise store data to receive in the receive
@@ -468,9 +456,8 @@ fsp_err_t R_SSI_WriteRead (i2s_ctrl_t * const p_ctrl,
     FSP_ASSERT(NULL != p_dest);
     FSP_ERROR_RETURN(SSI_PRV_OPEN == p_instance_ctrl->open, FSP_ERR_NOT_OPEN);
 
-    /* bytes must be a non-zero multiple of the FIFO access size multiplied by 2 (for the left and right channel). */
+    /* bytes must be a non-zero */
     FSP_ASSERT(bytes > 0U);
-    FSP_ASSERT(0U == (bytes % (2U << p_instance_ctrl->fifo_access_size)));
 #endif
 
     /* If a transfer instance is provided for write, reset the transfer. Reset the transmit FIFO first since the
@@ -625,25 +612,6 @@ fsp_err_t R_SSI_Close (i2s_ctrl_t * const p_ctrl)
     return FSP_SUCCESS;
 }
 
-/***********************************************************************************************************************
- * DEPRECATED Sets driver version based on compile time macros.
- *
- * @retval     FSP_SUCCESS          Successful close.
- * @retval     FSP_ERR_ASSERTION    The parameter p_version is NULL.
- **********************************************************************************************************************/
-fsp_err_t R_SSI_VersionGet (fsp_version_t * const p_version)
-{
-#if SSI_CFG_PARAM_CHECKING_ENABLE
-
-    /* Verify parameters are valid */
-    FSP_ASSERT(NULL != p_version);
-#endif
-
-    p_version->version_id = g_version.version_id;
-
-    return FSP_SUCCESS;
-}
-
 /*******************************************************************************************************************//**
  * Updates the user callback and has option of providing memory for callback structure.
  * Implements i2s_api_t::callbackSet
@@ -823,6 +791,10 @@ static void r_ssi_stop_sub (ssi_instance_ctrl_t * const p_instance_ctrl)
 
     p_instance_ctrl->p_reg->SSICR = ssicr;
 
+    uint32_t ssifcr = p_instance_ctrl->p_reg->SSIFCR;
+    ssifcr &= ~(R_SSI0_SSIFCR_TIE_Msk | R_SSI0_SSIFCR_RIE_Msk);
+    p_instance_ctrl->p_reg->SSIFCR = ssifcr;
+
 #if SSI_CFG_DTC_ENABLE
 
     /* If transfer is used, disable transfer when stop is requested. */
