@@ -1,7 +1,12 @@
 // Important: RF *must* be activated on ESP board
 // https://techtutorialsx.com/2017/12/22/esp32-arduino-random-number-generation/
+#ifdef ESP32
+#    include <esp_system.h>
+#endif
 
-#include <esp_system.h>
+#ifdef ARDUINO
+#    include <Arduino.h>
+#endif
 
 static int
 hydro_random_init(void)
