@@ -1,2 +1,6 @@
+class ABC:
+    pass
+
+
 def abstractmethod(f):
     return f
